#ifndef FE_IS_INCLUDED
#define FE_IS_INCLUDED


#include <tools.h> // Lots of the tools we need is currently in tools.h. 
		   // These are functions on a lower level than the finite elements, 
		   // e.g. functions for making polynomial spaces

#include <Polygon.h>



class FE {
  public: 
  FE() {}
  ~FE() {}

  virtual void set(Polygon& p) = 0;     // Set polygonal domain 
  virtual Polygon& getPolygon() = 0;    // Get polygonal domain 
  virtual GiNaC::ex N(int i) = 0;      // The i'th basis function
  virtual GiNaC::ex dof(int i) = 0 ;   // The i'th degree of freedom 
  virtual int nbf() = 0;                // Number of basis functions/
                                        // degrees of freedom  

  // print and debug functions (need some thought) ??    
  // if the internal structures are public and general 
  // STL objects, then such functions are not necessarilly needed
};

class StandardFE : public FE {
//  protected: 
  public: 
  GiNaC::exvector Ns; 
  GiNaC::exvector dofs; 
  Polygon* p; //FIXME should this be a reference ?  
  int order; 

  //a rough implementation of this element is in spectral.cpp, 
  //but there the reference element is hard coded 
  public: 
  StandardFE() {}
  ~StandardFE() {}

  virtual void set(int order); 
  virtual void set(Polygon& p);  
  virtual Polygon& getPolygon();  
  virtual void compute_basis_functions();  
  virtual int nbf(); 
  virtual GiNaC::ex N(int i);  
  virtual GiNaC::ex dof(int i); 
}; 



#endif

  






